#------------------------------------------------------------------------------
# snackAmp - minPlayer package 1.0 for snackAmp
#
# Music File Player in Tcl/Tk
#
# Copyright (C) 2001 Tom Wilkason
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
#
# Please send any comments/bug reports to
# tom.wilkason@cox.net  (Tom Wilkason)
#------------------------------------------------------------------------------
set [file rootname [file tail [info script]]]_History {
   $Header: /cvsroot/snackamp/snackamp/lib/miniPlayer.tcl,v 1.36 2009/03/14 14:31:40 wilkason Exp $
}
#----------------------------------------------------------------------------
# Namespace for miniPlayer
#----------------------------------------------------------------------------
namespace eval miniPlayer {
   variable mpSS ""
   variable splash
   variable Hidden 0
   variable Pinned 0
   variable lastFolderName ""
   variable imageLabel
   variable coverArt
   variable activeMiniPlayer
   variable fg
   variable bg
   variable efg
   variable ebg
   variable tsbg
   variable lastTrack ""
   upvar #0 snackAmpSettings(miniArt) coverArt
   upvar #0 snackAmpSettings(activeMiniPlayer) activeMiniPlayer
   upvar #0 snackAmpSettings(bg) bg
   upvar #0 snackAmpSettings(fg) fg
   upvar #0 snackAmpSettings(efg) efg
   upvar #0 snackAmpSettings(ebg) ebg
   upvar #0 snackAmpSettings(tsbg) tsbg
   option add *MiniPlayer*Label.background $bg
   option add *MiniPlayer*Button.background $bg
}
#------------------------------------------------------------------------------
# Function   : launch
# Description: Mini-player that pops up from the taskbar
# Author     : Tom Wilkason
# Date       : 2/1/2001
#------------------------------------------------------------------------------
proc miniPlayer::launch {X Y} {
   variable coverArt
   variable imageLabel
   variable activeMiniPlayer 1
   variable fg
   variable bg
   variable efg
   variable ebg
   variable tsbg
   variable imageSize
   variable lastTrack
   set w .miniPlayer
   if {[winfo exists $w]} {
      if {$::TA(currentMF) ne $lastTrack} {
         miniPlayer::smallCoverArt $::TA(currentMF)
      }
      wm deiconify $w
      fade::in $w
      focus -force $w.mpframe.tb.tt.title
      return
   }
   ##
   # Create miniPlayer window with no border, we will handle dragging
   #

   toplevel $w -relief raised -borderwidth 3 -class MiniPlayer  -background $bg
   settings::regColor $w -background ::snackAmpSettings(bg)

   wm focusmodel $w active  ;# Does this force a focus on pop-up?
   # Bottom Frame
   set TB $w.mpframe.tb
   set TT $w.mpframe.tb.tt
   set CF $w.aaframe
   pack [frame $CF -background $bg -borderwidth 0 -relief sunken] \
      -expand false -fill x -side left
   # Song Title
   pack [frame $w.mpframe -background $bg] -expand false -fill x -side left
   pack [frame $TB -background $bg] -expand false -fill x -side top -pady 2
   pack [frame $TT -background $bg] -expand true -fill x -side left
   settings::regColor $w.mpframe -background ::snackAmpSettings(bg)
   settings::regColor $CF -background ::snackAmpSettings(bg)
   settings::regColor $TB -background ::snackAmpSettings(bg)
   settings::regColor $TT -background ::snackAmpSettings(bg)

   label $TT.title -textvariable ::TA(playlistEntry) \
      -bg $bg  -bd 0 -fg $fg -font songInfo -width 45 \
      -anchor w -relief sunken -cursor fleur
   settings::regColor $TT.title -bg ::snackAmpSettings(bg) \
      -fg ::snackAmpSettings(fg)
   pack $TT.title -expand true -fill x -side top -padx 4

   # Time Slider
   set miniPlayer::mpSS [imgSlider::create $TT.ss \
      -to 100 \
      -image saImages.timeSlider \
      -hlimage saImages.timeSlider.hl \
      -background $bg \
      -fillcolor $tsbg \
      -troughheight 6 \
      -padding 2 \
      -depress {set ::TA(setdrag) 0}\
      -command {gui::updateTime} \
      -release {
            set ::TA(setdrag) 1
            PlayOffset
       } \
      -variable gui::trackPosition]

   set gui::mptsScale $miniPlayer::mpSS
   settings::regColor $miniPlayer::mpSS -background ::snackAmpSettings(bg) \
      -fillcolor ::snackAmpSettings(tsbg)
   balloon::define $miniPlayer::mpSS "Show/Adjust playback location within file"
   pack $miniPlayer::mpSS -expand true -fill x -side top -padx 4

   # Close miniPlayer
   button $TB.close -command {miniPlayer::close} \
      -image saImages.Close \
      -padx 0 -pady 0 \
      -borderwidth 0 \
      -background $bg
   #balloon::button $w.tb.close "Close mini-player"
   settings::regColor $TB.close -background ::snackAmpSettings(bg)
   imageTools::button $TB.close "Close mini-player"
   pack $TB.close -side right -padx 0 -pady 0
   set BF $w.mpframe.frame
   frame $BF -background $bg
   settings::regColor $BF -background ::snackAmpSettings(bg)

   set volScale [imgSlider::create $BF.volume \
      -from $::snackAmpSettings(minDB) -to 0 \
      -image saImages.volSlider \
      -hlimage saImages.volSlider.hl \
      -background $bg \
      -fillcolor  $tsbg \
      -troughheight 6 \
      -padding 0 \
      -command {PlayGain} \
      -release {PlayGain $::soundControl::acGaindB} \
      -variable soundControl::acGaindB]

   balloon::define $volScale "Adjust the playback level"

   settings::regColor $volScale -background ::snackAmpSettings(bg) \
      -fillcolor ::snackAmpSettings(tsbg)
   # Reconnect the volume
   bind $volScale <ButtonPress-1> {
      soundControl::initVolume
   }

   pack $BF -padx 4 -pady 2 -fill x -expand true
   pack $volScale -expand true -fill x -side left
   balloon::widgetDefine $TT.title

   # Since this is override direct, we have to handle drags
   bind  .miniPlayer <Enter> [list
      focus -force $TT.title
      wm deiconify .miniPlayer
   ]
   # Pop-up info on the song
   if {$::tcl_platform(platform) != "windows"} {
      bind .miniPlayer <Button-5> [list bumpLevel -1]
      bind .miniPlayer <Button-4> [list bumpLevel 1]
   } else {
      bind .miniPlayer <MouseWheel> {
         set amount [expr {%D/abs(%D)*1}] ;# get +/- 2
         bumpLevel $amount
      }
   }
   bind $TT.title <ButtonPress-1> [list miniPlayer::drag_start %W %X %Y]
   bind $TT.title <B1-Motion>     [list miniPlayer::drag_motion %W %X %Y]
   # System Menu on RMB
   if {$::snackAmpSettings(fancyMenu)==0} {
      bind  $TT.title <ButtonPress-3> {
         destroy .miniPlayer.mpframe.tb.tt.title.balloon
         tk_popup $::SA.bRmenubar %X %Y
      }
   }

   # Buttons for audio control
   set buttonData {
      image player_start   Prev       "Play previous in list"
      image player_play    PlayResume "Play Selection"
      image player_pause   Pause      "Pause/Resume playback"
      image player_stop    Stop       "Stop playing"
      image player_next    Next       "Play next in list"
   }
   # build each one
   foreach {bmType File Command Help} $buttonData {
      button  $BF.$File -$bmType "saImages.$File" -command $Command \
         -borderwidth 0 -background $bg
      pack $BF.$File -side left -padx 0 -pady 0
      balloon::button $BF.$File $Help
      settings::regColor $BF.$File -background ::snackAmpSettings(bg)
   }

   # Rating
   set pm $BF.rating.popup
   pack [menubutton $BF.rating -relief flat -image saImages.iconthumbsup -menu $pm -background $bg]\
    -side left -padx 0 -pady 0
   set m [menu $pm \
   -postcommand [list gui::ratingPopup $pm]]

   settings::regColor $BF.rating -background ::snackAmpSettings(bg)   
   balloon::button $BF.rating "Adjust rating of the playing track, the more stars the better."

   # Support hiding of main window
   pack [checkbutton $BF.hide -background $bg \
      -selectimage saImages.arrowhide -image saImages.arrowshow -indicatoron 0 -bd 0 \
      -selectcolor $bg -variable miniPlayer::Hidden -command {
      switch -- [wm state $::SA] {
         normal {wm withdraw $::SA; set miniPlayer::Hidden 1}
         withdrawn {wm deiconify $::SA; set miniPlayer::Hidden 0}
         iconic {wm withdraw $::SA; set miniPlayer::Hidden 1}
         default {wm deiconify $::SA; set miniPlayer::Hidden 0}
      }
   }] -side left
   balloon::define $BF.hide "Hide/Show the main snackAmp window."
   settings::regColor $BF.hide -background ::snackAmpSettings(bg)
   # Icon to pin it if can pin
   if {$ontop::enabled} {
      pack [checkbutton $BF.pin -image saImages.unpinit -selectimage saImages.pinit -indicatoron 0 -bd 0 \
         -selectcolor $bg -background $bg\
         -variable miniPlayer::Pinned \
         -command {ontop::control .miniPlayer $miniPlayer::Pinned}] -side left

      settings::regColor $BF.pin -background ::snackAmpSettings(bg)   
      balloon::define $BF.pin "Press to keep the mini player on top."
      #----------------------------------------------------------------------------
      # Transparency button
      #----------------------------------------------------------------------------
      fade::setDefTrans .miniPlayer
      pack [menubutton $BF.trb -image saImages.transparent -menu $BF.trb.tr \
         -relief flat -bd 0 -background $bg] -side left

      settings::regColor $BF.trb -background ::snackAmpSettings(bg)   
      balloon::button $BF.trb "Select transparency level for mini player"
      menu $BF.trb.tr
      for {set i 0.3} {$i <= 1.01} {set i [expr {$i+0.05}]} {
         if {$i>0.951} {
            set i 1.0
            set lab "Normal"
         } else {
            set lab "[format %0.0f [expr {$i*100}]]%"
         }
         $BF.trb.tr add radiobutton -label $lab -value $i\
            -variable snackAmpSettings(.miniPlayer,transparency) \
            -command [list fade::setTrans .miniPlayer $i]
      }
   }
   # This is just clutter
   if {0} {
      #----------------------------------------------------------------------------
      # Search frame
      #----------------------------------------------------------------------------
      set sf [frame $w.search]
   
      set sf0 [label $sf.lab -text "Search:"]
      set sf1 [entryHistory::create $sf.search ::TA(filter) ::snackAmpSettings(History)]
      set sf1h [entryHistory::hull $sf.search]
      $sf1h config -background $ebg -foreground $efg
      balloon::define $sf1h "Enter text to search for, use buttons or Return for playlist search and Control-Return for Media Library search"
   
      set sf2 [button $sf.b1 -image saImages.findImage -command {
         if {[wm state $::SA]!="normal"} {
            wm deiconify $::SA
            set miniPlayer::Hidden 0
         }
         cframe.tryOpen $::TA(CF)
         findNextSong
      }]
      balloon::button $sf.b1 "Search Playlist"
   
      set sf3 [button $sf.b2 -image saImages.filterOff -command {
         filtering::searchFilter
      }]
      balloon::button $sf.b2 "Search Media Library"
   
      bind $sf1h <Return> [list $sf.b1 invoke]
      bind $sf1h <Control-Return> [list $sf.b2 invoke]
   
      grid $sf0 $sf1 $sf2 $sf3 -sticky ew
      grid columnconfigure $sf 1 -weight 1
      pack $sf -side top -fill x -expand true
   }
   #----------------------------------------------------------------------------
   # Cover art frame
   #----------------------------------------------------------------------------
   if {$coverArt} {
      # Cover art above window
      set imageLabel [label $CF.cover -bd 1 -relief sunken]
      pack $imageLabel -expand true -fill both
      pack $CF -side left -expand true -fill both
      settings::regColor $imageLabel -background ::snackAmpSettings(bg)
   }
   ##
   # Figure out where to put the window, TL corner is window 0,0
   #
   # Hide it and take away window decoration
   wm withdraw $w
   wm overrideredirect $w 1
   focus -force $w
   # with it hidden, let the layout take place so we know its geometry
   update idletasks

   # Get miniplayer size  and position at bottom right of usable area
   set rX [winfo reqheight $w]
   # Save the required image size and keep it constant
   if {![info exists imageSize]} {
      set imageSize $rX
   }
   if {$coverArt} {
      miniPlayer::smallCoverArt [soundControl::fileName]
      update idletasks
      bind $imageLabel <ButtonPress-1> [list miniPlayer::drag_start %W %X %Y]
      bind $imageLabel <B1-Motion>     [list miniPlayer::drag_motion %W %X %Y]

   }
   set rY [winfo reqheight $w]
   # Don't place over taskbar
   # Locate the player in the right hand bottom corner then pop it up.
   if {($X+$Y)==0} {
      set x [expr {$::TA(screen_right)-$rX}]
      set y [expr {$::TA(screen_bottom)-$rY}]
   } else {
      set x $X
      set y $Y
   }
   wm geometry $w "+$x+$y"
   set ::snackAmpSettings(positionMiniPlayer) [list $x $y]
   fade::off $w
   wm deiconify $w
   if {$::tcl_platform(os) eq "Linux"} {
      # yuck: linux bug workaround
      wm geometry $w "+$x+$y"
   }
   fade::in $w
}
#----------------------------------------------------------------------------
# Cover art for the mini player
#----------------------------------------------------------------------------
proc miniPlayer::smallCoverArt {track {size 0}} {
   Trace
   variable imageLabel
   variable coverArt
   variable lastFolderName
   variable imageSize
   variable lastTrack
   if {$coverArt} {
      set sourceImage [albumart::setCurrentImage $track]
      set folder [file dirname $track]
      if {$folder ne $lastFolderName} {
         if {$sourceImage ne "" && [albumart::valid]} {
            image create photo "saImages.miniCoverArt" ;# this replaces old version
            if {$size==0} {
               set height $imageSize
            } else {
               set height $size
            }
            foreach {height width} \
               [albumart::getCoverDims $sourceImage $height] {break}
            if {$width>0 && $height>0} {
               set lastTrack $track

               tkImageTools::resize $sourceImage "saImages.miniCoverArt" $width $height
               $imageLabel configure -image "saImages.miniCoverArt"
            } else  {
               $imageLabel configure -image ""
            }
         } else {
            $imageLabel configure -image ""
         }
         set lastFolderName $folder
      }
   }
}
#------------------------------------------------------------------------------
# Function   : miniPlayer::mpTime
# Description: Show time on miniplayer when dragging mouse
# Author     : Tom Wilkason
#------------------------------------------------------------------------------
proc miniPlayer::mpTime {W X Y} {
   set time [HMSfrac $gui::trackPosition]
   balloon::show $W $X $Y "Time: $time"
}
##
# Motion start
#
proc miniPlayer::drag_start {W X Y} {
   variable splash
   set W [winfo toplevel $W]
   set splash(X) [expr {$X - [winfo x $W]}]
   set splash(Y) [expr {$Y - [winfo y $W]}]
   destroy .miniPlayer.mpframe.tb.tt.title.balloon
}
##
# Motion process
#
proc miniPlayer::drag_motion {W X Y} {
   variable splash
   set x [expr {$X - $splash(X)}]
   set y [expr {$Y - $splash(Y)}]
   wm geometry [winfo toplevel $W] "+$x+$y"
   set ::snackAmpSettings(positionMiniPlayer) [list $x $y]
   destroy .miniPlayer.mpframe.tb.tt.title.balloon
}
#------------------------------------------------------------------------------
# Function   : miniPlayer::toggle
# Description: Toggle the mini player visibility
# Author     : Tom Wilkason
#------------------------------------------------------------------------------
proc miniPlayer::toggle {{X 0} {Y 0}} {
   variable activeMiniPlayer
   if {$activeMiniPlayer == 0} {
      miniPlayer::close
   } else {
      miniPlayer::launch $X $Y
   }
}
#----------------------------------------------------------------------------
# Version that doesn't rely on the activeMiniPlayer bit being set first
#----------------------------------------------------------------------------
proc miniPlayer::toggler {{X 0} {Y 0}} {
   variable activeMiniPlayer
   if {$activeMiniPlayer == 1} {
      miniPlayer::close
      set activeMiniPlayer 0
   } else {
      miniPlayer::launch $X $Y
      set activeMiniPlayer 1
   }
}

#------------------------------------------------------------------------------
# Function   : miniPlayer::close
# Description: Close the miniplayer and make sure main window is not also
#              withdrawn on non-windows systems (i.e. no task bar)
# Author     : Tom Wilkason
#------------------------------------------------------------------------------
proc miniPlayer::close {args} {
   variable activeMiniPlayer 0

   set w .miniPlayer
   # todo: need blocking fadeout (have code)
   fade::out $w
   # If no taskbar icon, then make sure both windows
   # not closed.
   if {$::TA(iconsVisible)==0} {
      if {[wm state $::SA] == "withdrawn"} {
         wm deiconify $::SA
         set miniPlayer::Hidden 0
      }
   }
}
package provide miniPlayer 1.0
