##################################################################################
#                             m i n i c o n t r o l
##################################################################################
##
# Return the playing status
#
proc remote/mini::status {} {
   if {[snack::audio active]} {
      if {$soundControl::playState!="Stopped"} {
         if {$soundControl::paused==1} {
            set pre "Paused"
         }  else  {
            set pre "Playing"
         }
         set vol [expr {int([soundControl::getGlobalGain])}]
         return "$pre [_calctime [soundControl::position]] / [_calctime [soundControl::duration]] (vol=$vol %)"
      }  else  {
         return "Stopped"
      }
   }
   return "Not playing anything"
}
##
# Return the control buttons
#
proc remote/mini::buttons {} {
   set rc ""
   return {<center><form action="/remote/mini/" method="get"><input type="image" name="panel" src="/remote/mini/buttons.gif" width="170" height="28"></form></center>}
}
##
# Return the current playlist
#
proc remote/mini::playlist {styleN styleA cnt} {
   set cnt0 [expr {$cnt/2}]
   set cnt1 [expr {$cnt-$cnt0-1}]
   set cur [LB::idxPlaying]
   if {$cur<0} {set cur 0}
   set end [LB::idxEnd]
   if {$cur<$cnt0} {
      incr cnt1 [expr {$cnt0-$cur}]
      set cnt0 $cur
   }
   if {($cur+$cnt1)>=$end} {
      set c [expr {($cur+$cnt1)-$end+1}]
      incr cnt1 -$c
      incr cnt0 $c
      if {$cur<$cnt0} {
         set cnt0 $cur
      }
   }
   set rc "<table width=\"100%\" border=\"0\" cellspacing=\"0\" cellpadding=\"1\">\n"
   set pos [expr {$cur-$cnt0}]
   set  [LB::idxPlaying]
   for {set i 0} {$i<($cnt0+$cnt1+1)} {incr i} {
      if {$pos==$cur} {
         set cls $styleA
         set a0 "<i>"
         set a1 "</i>"
      }  else  {
         set cls [lindex $styleN [expr {$i%[llength $styleN]}]]
         set a0 "<a href=\"/remote/mini/?playId=$pos\">"
         set a1 "</a>"
      }
      append rc "<tr><td height=\"14\" class=\"$cls\" onclick=\"location.href='/remote/mini/?playId=$pos'\">$a0[string replace [LB::keyFromIndex $pos] 100 end ""]$a1<br></td></tr>\n"
      incr pos
   }
   append rc "</table>"
   return $rc
}
##################################################################################
#                             URL Processor
##################################################################################

proc remote/mini::doit {} {
   #Trace 1
   upvar data data
   set args [split $data(query) &]
   eval [concat [list HttpD::GetParams {playId "" cmd "" volume "" panel.x "" panel.y "" filter "" gofilter ""}] $args]
   set rc ""
   if {$playId!=""} {
      set n [LB::keyFromIndex $playId]
      if {$n!=""} {
         LB::activate $playId; Play active
      }
   }
   # Lay on commands for the buttons.gif image
   set px ${panel.x} 
   set py ${panel.y}
   if {$px!=""} {
      if {$py<16} {
         if {$px<80} {
            set cmd [lindex {prev stop pause play next} [expr {$px/16}]]
         } elseif {$px>=104} {
            set volume [expr {int(($px-106)*50.0/63.0)-50}]   ;#169-106=63 max , want -50 t0 0 on scale
         }
      }  else  {
         set pos [expr {round(double([soundControl::length])*double($px)/170.0)}]
         PlayOffset $pos
      }
   }
   if {($gofilter!="")||($filter!="")} {
      set ::TA(filter) $filter
      filterMusic replace
      return
   }
   if {$cmd!=""} {
      switch -- $cmd {
         prev {
            Prev; update
         }
         next {
            Next; update
         }
         play {
            Play
         }
         pause {
            Pause
         }
         stop {
            Stop
         }
      }
   }
   if {$volume!=""} {
      PlayGain $volume
   }
   return ""
}


proc remote/mini::_calctime {t} {
   set t [expr {round($t)}]
   return [format "%d:%02d" [expr {$t/60}] [expr {$t%60}]]
}

##################################################################################
#                             s o n y   p s p   c o n t r o l
##################################################################################
# todo:
# clean up parameters to be like other do routines
# add buttons for ratings
# remove handling of px.x/y handling since no longer using single image
# make buttonProcess a common function to other uses
proc remote/psp::doit {} {
   upvar data data
   set args [split $data(query) &]
   eval [concat [list HttpD::GetParams {bt_Control "" playId "" cmd "" volume "" panel.x "" panel.y "" filter ""}] $args]
   set rc ""
   if {$playId!=""} {
      set n [LB::keyFromIndex $playId]
      if {$n!=""} {
         LB::activate $playId; Play active
      }
   }
   # todo: Add code to rate track on image
   # Lay on commands for the buttons.gif image
   # buttons are 16 x 16 in size
   set px ${panel.x};
   set py ${panel.y}; 
   if {$px!=""} {
      if {$py<16} {
         set cmd [lindex {prev stop pause play next plus minus 1 2 3 4 5 0} [expr {$px/16}]]
      }
   }
   ;# process indiv buttons
   if {[string match "bt*" $args]} {
      buttonProcess $args
   } elseif {$cmd!=""} {
      switch -- $cmd {
         prev {
            Prev; update
         }
         next {
            Next; update
         }
         play {
            Play
         }
         pause {
            Pause
         }
         stop {
            Stop
         }
         refresh {
            
         }
         plus {
            if {[catch {::macros::bumpLevel 2} result] } then {
               saLog $result
            }
         }
         minus {
            if {[catch {::macros::bumpLevel -2} result] } then {
               saLog $result
            }
         }
         0 - 
         1 - 
         2 -
         3 -
         4 -
         5 {
            # Set the track rating
            # Force a re-read of database for this track to display updates
            if {$cmd > 0} {
               set rating [string repeat * $cmd] 
            } else {
               set rating "--"
            }
            db::setVal [soundControl::fileName] Rating $rating
            trackList::updateFiles [list [soundControl::fileName]]
            set ::songArray(Rating) [mrating $rating]
         }
      }
   }
   if {$volume!=""} {
      PlayGain $volume
   }
   return ""
}

#------------------------------------------------------------------------------
# process a button press
#------------------------------------------------------------------------------
proc remote/psp::buttonProcess {param} {
   foreach {CmdType Cmd} [split $param =] {break}
   set Cmd [HttpD::Decode $Cmd]
   switch -- $CmdType {
      bt_Control {
         if {[catch {::macros::$Cmd} result] } then {
            append Result "<p>$::errorInfo</p>\n"
         }
      }
      bt_skip {
         if {[catch {::macros::skipPlay $Cmd} result] } then {
            append Result "<p>$::errorInfo</p>\n"
         }
      }
      bt_vote {
         # TODO: Add code to increment the vote count
         # Force a re-read of database for this track to display updates
         HTH::clearCache [soundControl::fileName]
      }
      bt_rating {
         # Set the track rating
         # Force a re-read of database for this track to display updates
         HTH::clearCache [soundControl::fileName]
         db::setVal [soundControl::fileName] Rating $Cmd
         trackList::updateFiles [list [soundControl::fileName]]
         set ::songArray(Rating) [mrating $Cmd]
      }
      bt_track_name {
         # step 1
         set bt_track_name $Cmd
      }
      bt_track_rating {
         # step 2
         # Set the track rating for some track
         HTH::clearCache $bt_track_name
         db::setVal $bt_track_name Rating $Cmd
         trackList::updateFiles [list $bt_track_name]
         if {[soundControl::fileName] eq $bt_track_name} {
            set ::songArray(Rating) [mrating $Cmd]
         }

      }
      bt_Level {
         if {[catch {::macros::bumpLevel $Cmd} result] } then {
            append Result "<p>$::errorInfo</p>\n"
         }
      }
      play {
         # Play a specific song
         # /remote/control?play=some+name.mp3
         if {[catch {playSpecificSong $Cmd} error] } then {
            debug $result
            append Result $result
         }
      }
      delete {
         LB::deleteIndex $Cmd
         gui::updateFileDisplay
         # Remove from playlist
      }
      cmb_APL {
         # Select an AutoPlaylist file
         # TFW: If a real + is in file name, this may fail
         after 200 [list macros::readAPLFile [file join $::HOME $Cmd.tafs] 1]
      }
      cmb_Sort {
         # Sort/Shuffle the files
         set ::snackAmpSettings(sortModeTxt) $Cmd
         set subSort::sortSelection $::snackAmpSettings(sortModeTxt)
         # Hack, if a file then a custom sort must be read in
         set fname [file join $::HOME $Cmd.ss]
         if {[file exists $fname]} {
            subSort::readFile $Cmd
         }
         after 200 filtering::sortPlaylist
      }
      filter_text {
         # stage 1
         set ::TA(filter) $Cmd
         # Note: These will happen in this order, search_replace can process
         # Button=Filter&filter_text=ab&filter_what=All+Files&search_replace=Replace
      }
      filter_what {
         # stage 2
         switch -glob -- [string tolower $Cmd] {
            "all*files" {set Search_What 1}
            "play*" {set Search_What playList}
            "auto*" {set Search_What 0}
            default {set Search_What 1}
         }
      }
      search_replace {
         # stage 3, do the filtering
         if {![info exists Search_What]} {set Search_What 1}
         set Search_Replace [string tolower $Cmd]
         filtering::pushFilter
         macros::filterMusic $Search_Replace $Search_What
         # stage 3
      }
      cmb_PL {
         # Stage 1
         # Set the playlist file
         foreach ext {m3u taf} {
            set Playlist_File [file join $::HOME $Cmd.$ext]
            if {[file exists $Playlist_File]} {
               break
            }
         }
      }
      PL_search_replace {
         # Stage 2
         # And then apply it
         macros::ProcessPlaylistFile $Playlist_File [string tolower $Cmd] 0
      }
      bt_Submit {
         # This isn't handled specifically, it comes first
      }
      default {}
   }
}
proc remote/psp::_calctime {t} {
   set t [expr {round($t)}]
   return [format "%d:%02d" [expr {$t/60}] [expr {$t%60}]]
}

proc remote/psp::status {} {
   if {[snack::audio active]} {
      if {$soundControl::playState!="Stopped"} {
         if {$soundControl::paused==1} {
            set pre "Paused"
         }  else  {
            set pre "Playing"
         }
         set vol [expr {int([soundControl::getGlobalGain])}]
         set trackInfo [remote/psp::trackInfo  [soundControl::fileName] 2]
         return "$trackInfo<br>$pre [_calctime [soundControl::position]] / [_calctime [soundControl::duration]] (Vol=$vol %)"
      }  else  {
         return "Stopped"
      }
   }
   return "Not playing anything"
}
proc remote/psp::trackInfo {fileName {short 1}} {
   variable InfoCache
   variable initDetails
   if {[info exists InfoCache($fileName)]} {
      array set Details $InfoCache($fileName)
   } else {
      # Init data with html blanks
      array set Details [array get initDetails]
      if {[db::lookup $fileName Details 1]<=0} {
         HttpD::Log "trackInfo:$error"
      }
      # Stuff not in the database
      set Details(Cover) [HTH::coverArtShadow [HTH::coverArt $Details(Folder) $::snackAmpSettings(coversize)]]
      set Details(Name) [file tail $fileName]
      set Details(Position) "N/A"
      # Fill in non tagged data based on file name
      foreach item {Title Artist Album} {
         if {$Details($item)=="&nbsp;"} {
            set Details($item) [id3Tag::guess${item} $fileName]
         }
      }
      ##
      # File info
      #
      if {![catch {file stat $fileName Stat}]} {
         set Details(fSize) "[commify $Stat(size)] Bytes"
         set Details(fDate) [timeFormat $Stat(mtime)]
      }
      # Store all this good stuff for fast access later
      set InfoCache($fileName) [array get Details]
   }
   # Don't keep it around forever
   HTH::cleanLater HTH::InfoCache($fileName) 10
   set rating [db::getVal $fileName Rating]
   if {[string is double -strict $rating]} {
      set rating [string repeat * $rating]
   }
   set result "<font size=+2>$rating<br>$Details(Title)</font><br>$Details(Artist)<br>$Details(Album)"
   return $result
}

proc remote/psp::art {} {
   # cover art image
   set result [HTH::coverArt [file dirname $::TA(currentMF)] 128]
   return $result
}
proc remote/psp::buttons {} {
   set rc ""
   set result {<form action="/remote/psp/" method="get"><input type="image" name="panel" src="/remote/psp/buttons_rating.gif" width="208" height="16"></form>}
   return $result
}
proc remote/psp::playlist {styleN styleA cnt} {
   set cnt0 2 ;# prior
   set cnt1 [expr {$cnt-$cnt0-1}]
   set cur [LB::idxPlaying]
   if {$cur<0} {set cur 0}
   set end [LB::idxEnd]
   if {$cur<$cnt0} {
      incr cnt1 [expr {$cnt0-$cur}]
      set cnt0 $cur
   }
   if {($cur+$cnt1)>=$end} {
      set c [expr {($cur+$cnt1)-$end+1}]
      incr cnt1 -$c
      incr cnt0 $c
      if {$cur<$cnt0} {
         set cnt0 $cur
      }
   }
   set rc "<table width=\"100%\" border=\"0\" cellspacing=\"0\" cellpadding=\"1\">\n"
   set pos [expr {$cur-$cnt0}]
   set cur [LB::idxPlaying]
   for {set i 0} {$i<($cnt0+$cnt1+1)} {incr i} {
      if {$pos==$cur} {
         set cls $styleA
         set a0 "<i>"
         set a1 "</i>"
      }  else  {
         set cls [lindex $styleN [expr {$i%[llength $styleN]}]]
         set a0 "<a href=\"/remote/psp/?playId=$pos\">"
         set a1 "</a>"
      }                                                                                                                                                ;#27
      append rc "<tr><td class=\"$cls\" onclick=\"location.href='/remote/psp/?playId=$pos'\">$a0[string replace [LB::keyFromIndex $pos] 80 end ...]$a1<br></td></tr>\n"
#      append rc "<tr><td height=\"14\" class=\"$cls\" onclick=\"location.href='/remote/psp/?playId=$pos'\">$a0[string replace [LB::keyFromIndex $pos] 80 end ...]$a1<br></td></tr>\n"
      incr pos
   }
   append rc "</table>"
   return $rc
}



package provide urlMinicontrol 1.0


